function [xt,ft, succ,numf,iarm,lambda] = armijo_search(prob, xc,fc,gc,lambda,plot_line)

% Armijo line search with polynomial interpolation
% Adapted from steep by C. T. Kelley, Dec 20, 1996

% xc, current point
% fc = F(sys,xc)
% gc is search direction
%
% returns: xt,ft: Armijo point and F(sys,xt)
%          succ = true if success
%          numf: nb of F evals
%          iarm: nb of Armijo backtrackings

if nargin == 5
    plot_line = false;
end

%%% constants
alp = 1e-4;
bhigh=.5; blow=.1;
%%%
MAX_IARM = 5;

numf = 0;
%
%       fixup for very long steps, see (3.50) in the book
%

%lambda = min(lambda,100/(1+sqrt(ip(xc,gc,gc)))); %norm(gc.Y,2)));

% trial evaluation at full step
xt = moveEIG(prob,xc,gc,lambda);
ft = F(prob,xt);
%ft = fun(prob,lambda,xc,gc);
numf = numf+1; iarm = 0;

fgoal = fc-alp*lambda*ip(xc,gc,gc); %norm(gc.Y,2)^2;
%
%       polynomial line search
%
q0=fc; qp0=-ip(xc,gc,gc); lamc=lambda; qc=ft;
if plot_line
    plot_Fline(prob,xc,gc,lambda,alp,fc)
    qp0
    pause
end

while(ft > fgoal)    
    iarm=iarm+1;
    if iarm==1
        lambda=polymod(q0, qp0, lamc, qc, blow, bhigh);
    else
        lambda=polymod(q0, qp0, lamc, qc, blow, bhigh, lamm, qm);
    end
    qm=qc; lamm=lamc; lamc=lambda;
    xt = moveEIG(prob,xc,gc,lambda);
    ft = F(prob,xt);
    %ft = fun(prob,lambda,xc,gc);
    numf = numf+1; qc=ft;
    if(iarm > MAX_IARM)
        succ = false;
        %plot_linesearch(sys,xc)        
        return
    end
    fgoal = fc-alp*lambda*ip(xc,gc,gc);
end
succ = true;


end

function plot_Fline(prob,xc,gc,lambda,alp,fc)
ft = []; fgoal = [];
ts = linspace(0,lambda*1.1,100);
for t = ts
    xt = moveEIG(prob,xc,gc,t);
    ft = [ft F(prob,xt)];
    alp = 1;
    fgoal = [fgoal fc-alp*t*ip(xc,gc,gc)];
end
figure(1)
clf
plot(ts,ft,'bo')
hold all
plot(lambda,F(prob,moveEIG(prob,xc,gc,lambda)),'ro')
hold all
plot(ts,fgoal,'k--')
end
