function [x,stats,fail] = gn_rank(prob,opts,x)
% Gauss-Newton solver
  
% Adapted from - Riemannian Trust-Region
% (C) 2004-2007, P.-A. Absil, C. G. Baker, K. A. Gallivan
% Florida State University
% School of Computational Science

   % set pointers for functions required by RTR
   n = prob.n1;
   k = prob.r;
   %%N = diag(n:-1:1);
   fns.R    = @(x,h)moveEIG(prob,x,h,1);      % retraction
   fns.g    = @(x,h1,h2)ip(x,h1,h2);      % Riemannian metric
   fns.proj = @(x,z)projTxM(x,z);   % projection onto tangent plane from nearby
   fns.f     = @(x)F(prob,x);     % objective function
   fns.fgrad = @(x)grad(prob,x);  % gradient of f
   fns.fhess = @(x,eta)hessVec_simp(prob,x,eta);     % Hessian of f
   fns.randT = @(x)randTxM(x);
   fns.scale = @(h,c)scaleTxM(h,c);
   fns.plus  = @(h1,h2,c1,c2)plusTxM(h1,h2,c1,c2);   
   fns.shrink_rank = @(x, fx)shrink_rank(prob, x, fx);
   %fns.prec  = @(x,eta)precondLocal(sys,x,eta);
   %fns.prec  = @(x,eta)precond_split(sys,x,eta);
   %fns.prec   = @(x,eta)precond_test(sys,x,eta);
   % set parameters for RTR algorithm
   
   
   params.x0 = x;
   
   
  
   
   if ~isfield(params,'Delta_bar')
       params.Delta_bar = size(x.V,1)*size(x.V,2);   
   end
   
   if ~isfield(params,'Delta0'),
      params.Delta0   = params.Delta_bar/8;
   end
   
   if ~isfield(params,'epsilon')
       params.epsilon = 0;%1e-6;
   end
   if ~isfield(params,'max_outer')
       params.max_outer = opts.maxit;
   end
   
   if ~isfield(params,'max_inner')
       params.max_inner = k*(2*n-k);
   end
   params.verbosity = opts.verbosity;
   
   [x,stats,fail] = rtr(fns,params,prob,opts);
   
   
    