function [model infos] = svp(data, model, params)
  
  m = model.d1;
  n = model.d2;
  k = model.r;
  
  p = length(data.ls.nentries)/(m*n); %sampling density

  if isfield(params,'maxiter'),
    params.mxitr = params.maxiter;
  end
  
  params = SetDefaultParams(params,p);
  
  success = false;
  nbAttempt = 10;
  
  while (~success && nbAttempt > 0),
  
      try

        [model infos] = svp_(model, data.ls.rows, data.ls.cols, data.ls.entries, m, n, k, params);
        success = true;

      catch exception,

        if (strcmp(exception.identifier,'ErrorSVP:Divergence')),
          % decrease step size
          fprintf('Divergence => decreasing step size\n');
          params.delta = params.delta * 2;
          params.step_size= 1 / ((1 + params.delta ) * p);
        else
          throw(exception);
        end

      end
      
      nbAttempt = nbAttempt - 1;
  
  end
  
  if (~success),
    error('Could not find an appropriate step size within %d attempt',nbAttempt);
  end
    
%   if ~isempty(data.ts.cols) && ~isempty(data.ts.rows),
%     bpreds = proj_omega_sub(model.U*diag(model.B), model.V, data.ts.rows, data.ts.cols);
%   else
%     bpreds = [];
%   end
end
