%Test script to run SVP for matrix completion using uniformly sampled random matrices
%Written by: Prateek Jain (pjain@cs.utexas.edu) and Raghu Meka (meka@cs.utexas.edu)
%Last updated: October, 2009

m = 1000;
n = 1000;
k = 10; %rank of the optimal matrix
p = 0.1; %fraction of known entries
params.tol=1e-3;
params.vtol=1e-10;
params.mxitr=1000;
params.verbosity=1;
params.decreaselr = false;

U = randn(m,k);
V = randn(n,k); %Generate the underlying matrix

[train, test] = crossvalind('HoldOut',m*n,1-p);

[rows_ls,cols_ls] = ind2sub([m,n],find(train));
[rows_ts,cols_ts] = ind2sub([m,n],find(test));

entries_ls = proj_omega_sub(U,V,rows_ls,cols_ls);

model.d1 = m;
model.d2 = n;
model.r = k;

params.delta = 1/3;

fprintf('Running SVP with m=%d, n=%d, p=%f...\n',m,n,p);
t=cputime;
[bpreds model infos] = svp(rows_ts,cols_ts, rows_ls,cols_ls, entries_ls, model, params);
t = cputime-t;

entries_ts = proj_omega_sub(U,V,rows_ts,cols_ts);


fprintf('SVP finished with RMSE=%f in time t=%f\n',norm(bpreds - entries_ts)/sqrt(length(entries_ts)),t);
